import altair as alt
import geopandas as gpd
import six
import json
import warnings
from toolz.curried import curry


def geopandas_to_dict(data):
    try:
        if ('geometry' != data.geometry.name) and ('geometry' in data.columns):
            warnings.warn(
                "Column name 'geometry' is reserved name for " +
                "GeoDataFrame. Column named 'geometry' should contain " +
                "actual displaying geometry or not be used. Data of column " +
                "will not be accessible from the chart description. "
            )
        if 'type' in data.columns:
            warnings.warn(
                "Column name 'type' is reserved name for GeoDataFrame. " +
                "Data of column 'type' will not be accessible from the " +
                "chart description."
            )
        if 'id' in data.columns:
            warnings.warn(
                "Column name 'id' is reserved name for GeoDataFrame for " +
                "index values. Data of column 'id' will not be accessible " +
                "from the chart description."
            )
        if data.crs:
            data = data.to_crs(epsg=4326)
        return [dict(
                    row,
                    type=feature['type'],
                    geometry=feature['geometry'],
                    id=feature['id']
                    ) for row, feature in zip(
                            data.drop(
                                data.geometry.name,
                                axis=1
                            ).to_dict('row'),
                            data.geometry.__geo_interface__['features']
                        )
                ]
    except AttributeError as err:
        if str(err).startswith('No geometry data set yet'):
            warnings.warn("GeoDataFrame has no geometry to display.")
            return data.to_dict('row')
        else:
            raise


@curry
def gpd_to_values(data):
    """Replace a GeoDataFrame by a data model with values.
    For ``geopandas.GeoDataFrame`` columns values are stored as Foreign Members 
    of GeoJSON feature objects. For all other types uses function
    :py:func:`altair.to_values`."""
    if isinstance(data, gpd.GeoDataFrame):
        data = alt.utils.sanitize_dataframe(data)
        values = geopandas_to_dict(data)
        return {'values': json.dumps(values)}
    else:
        return alt.to_values(data)


@curry
def gpd_to_json(data):
    """Write the data model to a .json file and return a url based data model.
    For ``geopandas.GeoDataFrame`` columns values are stored as Foreign Members 
    of GeoJSON feature objects. For all other types uses function
    :py:func:`altair.to_json`."""
    if isinstance(data, gpd.GeoDataFrame):
        data = alt.utils.sanitize_dataframe(data)
        values = geopandas_to_dict(data)
        return alt.to_json({'values': values})
    else:
        return alt.to_json(data)


alt.data_transformers.register(
    'gpd_to_values',
    lambda data: alt.pipe(data, alt.limit_rows, gpd_to_values)
)
alt.data_transformers.register(
    'gpd_to_json',
    lambda data: alt.pipe(data, alt.limit_rows, gpd_to_json)
)

alt.data_transformers.enable('gpd_to_values')


def geojson_feature(data, feature='features', **kwargs):
    """A convenience function for extracting features from a geojson object or url

    Parameters
    ----------

    data : anyOf(string, geojson.GeoJSON)
        string is interpreted as URL from which to load the data set.
        geojson.GeoJSON is interpreted as data set itself.
    feature : string
        The JSON property containing the GeoJSON object set to convert to
          a GeoJSON feature collection. For example ``features[0].geometry``.
    \**kwargs :
        additional keywords passed to JsonDataFormat

    """
    if isinstance(data, six.string_types):
        return alt.UrlData(
                    url=data,
                    format=alt.JsonDataFormat(
                        type='json',
                        property=feature,
                        **kwargs
                    )
                )
    elif hasattr(data, '__geo_interface__'):
        if isinstance(data, gpd.GeoDataFrame):
            data = alt.utils.sanitize_dataframe(data)
        return alt.InlineData(
                    values=data.__geo_interface__,
                    format=alt.JsonDataFormat(
                        type='json',
                        property=feature,
                        **kwargs
                    )
                )
    else:
        warnings.warn("data of type {0} not recognized".format(type(data)))
        return data



# flake8: noqa
__version__ = '0.1.0.dev0'

from .geodata import (
    gpd_to_values,
    gpd_to_json,
    geojson_feature
)


__all__ = (
    'gpd_to_values',
    'gpd_to_json',
    'geojson_feature'
)



# flake8: noqa

import io
import os
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

LONG_DESCRIPTION = """
gpdVega: GeoPandas_ support for Altair_

.. _Altair: http://altair-viz.github.io/
.. _GeoPandas: http://geopandas.org/
"""


# ==============================================================================
# Utilities
# ==============================================================================

def read(path, encoding='utf-8'):
    path = os.path.join(os.path.dirname(__file__), path)
    with io.open(path, encoding=encoding) as fp:
        return fp.read()


def get_install_requirements(path):
    content = read(path)
    return [
        req
        for req in content.split("\n")
        if req != '' and not req.startswith('#')
    ]


def version(path):
    """Obtain the packge version from a python file e.g. pkg/__init__.py

    See <https://packaging.python.org/en/latest/single_source_version.html>.
    """
    version_file = read(path)
    version_match = re.search(r"""^__version__ = ['"]([^'"]*)['"]""",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


HERE = os.path.abspath(os.path.dirname(__file__))


# From https://github.com/jupyterlab/jupyterlab/blob/master/setupbase.py,
#  BSD licensed
def find_packages(top=HERE):
    """
    Find all of the packages.
    """
    packages = []
    for d, dirs, _ in os.walk(top, followlinks=True):
        if os.path.exists(os.path.join(d, '__init__.py')):
            packages.append(os.path.relpath(d, top).replace(os.path.sep, '.'))
        elif d != top:
            # Do not look for packages in subfolders if current is not a package
            dirs[:] = []
    return packages

# ==============================================================================
# Variables
# ==============================================================================


DESCRIPTION         = "GpdVega GeoPandas and Altair intergation"
NAME                = "gpdvega"
PACKAGES            = find_packages()
AUTHOR              = "Ilia Timofeev"
AUTHOR_EMAIL        = "ilia.timofeev@gmail.com"
URL                 = 'https://iliatimofeev.github.io/gpdvega/'
DOWNLOAD_URL        = 'https://github.com/iliatimofeev/gpdvega'
LICENSE             = 'BSD 3-clause'
INSTALL_REQUIRES    = get_install_requirements("requirements.txt")
DEV_REQUIRES        = get_install_requirements("requirements_dev.txt")
VERSION             = version(os.path.join(NAME, '__init__.py'))


setup(name=NAME,
      version=VERSION,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      url=URL,
      download_url=DOWNLOAD_URL,
      license=LICENSE,
      packages=PACKAGES,
      install_requires=INSTALL_REQUIRES,
      extras_require={
        'dev': DEV_REQUIRES
      },
      classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'],
)
